@@ -164,10 +164,14 @@
 #'                               configure, update, filename, charset)}}{
 #'    Uploads a Spatialite database to a target dataStore
 #'  }
-##'  \item{\code{uploadAppschema(ws, ds, endpoint,
+#'  \item{\code{uploadAppschema(ws, ds, endpoint,
 #'                               configure, update, filename, charset)}}{
 #'    Uploads a appschema file to a target dataStore
 #'  }
+#'  \item{\code{uploadGeopackage(ws, ds, endpoint,
+#'                               configure, update, filename, charset)}}{
+#'    Uploads a GeoPackage file to a target dataStore                             
+#'  }
 #' }
 #' 
 #' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
@@ -185,7 +189,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
     getDataStores = function(ws){
       self$INFO(sprintf("Fetching list of datastores in workspace '%s'", ws))
       req <- GSUtils$GET(
-        self$getUrl(), private$user, private$pwd,
+        self$getUrl(), private$user,
+        keyring::key_get(service = private$keyring_service, username = private$user),
         sprintf("/workspaces/%s/datastores.xml", ws),
         verbose = self$verbose.debug)
       dsList <- NULL
@@ -221,7 +226,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
     getDataStore = function(ws, ds){
       self$INFO(sprintf("Fetching datastore '%s' in workspace '%s'", ds, ws))
       req <- GSUtils$GET(
-        self$getUrl(), private$user, private$pwd,
+        self$getUrl(), private$user,
+        keyring::key_get(service = private$keyring_service, username = private$user),
         sprintf("/workspaces/%s/datastores/%s.xml", ws, ds),
         verbose = self$verbose.debug)
       dataStore <- NULL
@@ -248,7 +254,7 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       req <- GSUtils$POST(
         url = self$getUrl(),
         user = private$user,
-        pwd = private$pwd,
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = sprintf("/workspaces/%s/datastores.xml", ws),
         content = GSUtils$getPayloadXML(dataStore),
         contentType = "text/xml",
@@ -268,7 +274,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       updated <- FALSE
       self$INFO(sprintf("Updating datastore '%s' in workspace '%s'", dataStore$name, ws))
       req <- GSUtils$PUT(
-        url = self$getUrl(), user = private$user, pwd = private$pwd,
+        url = self$getUrl(), user = private$user, 
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = sprintf("/workspaces/%s/datastores/%s.xml", ws, dataStore$name),
         content = GSUtils$getPayloadXML(dataStore),
         contentType = "application/xml",
@@ -290,7 +297,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       deleted <- FALSE
       path <- sprintf("/workspaces/%s/datastores/%s.xml", ws, ds)
       if(recurse) path <- paste0(path, "?recurse=true")
-      req <- GSUtils$DELETE(self$getUrl(), private$user, private$pwd,
+      req <- GSUtils$DELETE(self$getUrl(), private$user, 
+                            keyring::key_get(service = private$keyring_service, username = private$user),
                             path = path, verbose = self$verbose.debug)
       if(status_code(req) == 200){
         self$INFO("Successfully deleted datastore!")
@@ -315,7 +323,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       }
       
       req <- GSUtils$GET(
-        self$getUrl(), private$user, private$pwd,
+        self$getUrl(), private$user,
+        keyring::key_get(service = private$keyring_service, username = private$user),
         sprintf("/workspaces/%s/datastores/%s/featuretypes.xml?list=%s", ws, ds, list),
         verbose = self$verbose.debug)
       ftList <- NULL
@@ -345,7 +354,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
     getFeatureType = function(ws, ds, ft){
       self$INFO(sprintf("Fetching featureType '%s' in datastore '%s' (workspace '%s')", ft, ds, ws))
       req <- GSUtils$GET(
-        self$getUrl(), private$user, private$pwd,
+        self$getUrl(), private$user, 
+        keyring::key_get(service = private$keyring_service, username = private$user),
         sprintf("/workspaces/%s/datastores/%s/featuretypes/%s.xml", ws, ds, ft),
         verbose = self$verbose.debug)
       featureType <- NULL
@@ -367,7 +377,7 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       req <- GSUtils$POST(
         url = self$getUrl(),
         user = private$user,
-        pwd = private$pwd,
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = sprintf("/workspaces/%s/datastores/%s/featuretypes.xml", ws, ds),
         content = GSUtils$getPayloadXML(featureType),
         contentType = "text/xml",
@@ -388,7 +398,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       self$INFO(sprintf("Updating featureType '%s' in datastore '%s' (workspace '%s')", featureType$name, ds, ws))
       updated <- FALSE
       req <- GSUtils$PUT(
-        url = self$getUrl(), user = private$user, pwd = private$pwd,
+        url = self$getUrl(), user = private$user, 
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = sprintf("/workspaces/%s/datastores/%s/featuretypes/%s.xml",
                        ws, ds, featureType$name),
         content = GSUtils$getPayloadXML(featureType),
@@ -411,7 +422,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       deleted <- FALSE
       path <- sprintf("/workspaces/%s/datastores/%s/featuretypes/%s.xml", ws, ds, ft)
       if(recurse) path <- paste0(path, "?recurse=true")
-      req <- GSUtils$DELETE(self$getUrl(), private$user, private$pwd,
+      req <- GSUtils$DELETE(self$getUrl(), private$user,
+                            keyring::key_get(service = private$keyring_service, username = private$user),
                             path = path, verbose = self$verbose.debug)
       if(status_code(req) == 200){
         self$INFO("Successfuly deleted featureType!")
@@ -430,7 +442,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
     getLayers = function(){
       self$INFO("Fetching layers")
       req <- GSUtils$GET(
-        self$getUrl(), private$user, private$pwd,
+        self$getUrl(), private$user, 
+        keyring::key_get(service = private$keyring_service, username = private$user),
         "/layers.xml", verbose = self$verbose.debug)
       lyrList <- NULL
       if(status_code(req) == 200){
@@ -459,7 +472,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
     getLayer = function(lyr){
       self$INFO(sprintf("Fetching layer '%s'", lyr))
       req <- GSUtils$GET(
-        self$getUrl(), private$user, private$pwd,
+        self$getUrl(), private$user,
+        keyring::key_get(service = private$keyring_service, username = private$user),
         sprintf("/layers/%s.xml", lyr),
         verbose = self$verbose.debug)
       layer <- NULL
@@ -479,7 +493,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       self$INFO(sprintf("Creating layer '%s'", layer$name))
       created <- FALSE
       req <- GSUtils$PUT(
-        url = self$getUrl(), user = private$user, pwd = private$pwd,
+        url = self$getUrl(), user = private$user,
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = sprintf("/layers/%s.xml", layer$name),
         content = GSUtils$getPayloadXML(layer),
         contentType = "application/xml",
@@ -500,7 +515,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       self$INFO(sprintf("Updating layer '%s'", layer$name))
       updated <- FALSE
       req <- GSUtils$PUT(
-        url = self$getUrl(), user = private$user, pwd = private$pwd,
+        url = self$getUrl(), user = private$user,
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = sprintf("/layers/%s.xml", layer$name),
         content = GSUtils$getPayloadXML(layer),
         contentType = "application/xml",
@@ -521,7 +537,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       self$INFO(sprintf("Deleting layer '%s'", lyr))
       deleted <- FALSE
       path <- sprintf("/layers/%s.xml", lyr)
-      req <- GSUtils$DELETE(self$getUrl(), private$user, private$pwd,
+      req <- GSUtils$DELETE(self$getUrl(), private$user,
+                            keyring::key_get(service = private$keyring_service, username = private$user),
                             path = path, verbose = self$verbose.debug)
       if(status_code(req) == 200){
         self$INFO("Successfuly deleted layer!")
@@ -544,7 +561,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
         self$INFO(sprintf("Fetching layer groups for workspace '%s'", ws))
       }
       req <- GSUtils$GET(
-        self$getUrl(), private$user, private$pwd,
+        self$getUrl(), private$user,
+        keyring::key_get(service = private$keyring_service, username = private$user),
         ifelse(missing(ws),"/layergroups.xml", sprintf("/workspaces/%s/layergroups.xml", ws)), 
         verbose = self$verbose.debug)
       lyrList <- NULL
@@ -578,7 +596,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
         self$INFO(sprintf("Fetching layer group '%s' in workspace '%s'", lyr, ws))
       }
       req <- GSUtils$GET(
-        self$getUrl(), private$user, private$pwd,
+        self$getUrl(), private$user,
+        keyring::key_get(service = private$keyring_service, username = private$user),
         ifelse(is.null(ws),
                sprintf("/layergroups/%s.xml", lyr),
                sprintf("/workspaces/%s/layergroups/%s.xml", ws, lyr)),
@@ -604,7 +623,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       }
       created <- FALSE
       req <- GSUtils$POST(
-        url = self$getUrl(), user = private$user, pwd = private$pwd,
+        url = self$getUrl(), user = private$user,
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = ifelse(is.null(ws),"/layergroups.xml",
                       sprintf("/workspaces/%s/layergroups.xml", ws)),
         content = GSUtils$getPayloadXML(layerGroup),
@@ -630,7 +650,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       }
       updated <- FALSE
       req <- GSUtils$PUT(
-        url = self$getUrl(), user = private$user, pwd = private$pwd,
+        url = self$getUrl(), user = private$user,
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = ifelse(is.null(ws),
                       sprintf("/layergroups/%s.xml", layerGroup$name),
                       sprintf("/workspaces/%s/layergroups/%s.xml", ws, layerGroup$name)),
@@ -659,7 +680,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       path <- ifelse(is.null(ws),
                      sprintf("/layergroups/%s.xml", lyr),
                      sprintf("/workspaces/%s/layergroups/%s.xml", ws, lyr))
-      req <- GSUtils$DELETE(self$getUrl(), private$user, private$pwd,
+      req <- GSUtils$DELETE(self$getUrl(), private$user, 
+                            keyring::key_get(service = private$keyring_service, username = private$user),
                             path = path, verbose = self$verbose.debug)
       if(status_code(req) == 200){
         self$INFO("Successfuly deleted layer group!")
@@ -735,7 +757,7 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
                      endpoint, paste0(supportedEndpoints, collapse=",")))
       }
       
-      supportedExtensions <- c("shp", "spatialite", "h2")
+      supportedExtensions <- c("shp", "spatialite", "h2", "gpkg")
       if(!(extension %in% supportedExtensions)){
         stop(sprintf("Unsupported extension '%s'. Possible values: [%s]",
                      extension, paste0(supportedExtensions, collapse=",")))
@@ -754,7 +776,8 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
       }
       
       req <- GSUtils$PUT(
-        url = self$getUrl(), user = private$user, pwd = private$pwd,
+        url = self$getUrl(), user = private$user,
+        pwd = keyring::key_get(service = private$keyring_service, username = private$user),
         path = sprintf("/workspaces/%s/datastores/%s/%s.%s?configure=%s&update=%s",
                        ws, ds, endpoint, extension, configure, update),
         content = NULL,
@@ -833,6 +856,18 @@ GSDataStoreManager <- R6Class("GSDataStoreManager",
                         configure, update, filename, charset,
                         contentType = "application/appschema")
       )
+    },
+    
+    #uploadGeoPackage
+    #---------------------------------------------------------------------------
+    uploadGeoPackage = function(ws, ds, endpoint = "file",
+                               configure = "first", update = "append",
+                               filename, charset = "UTF-8"){
+      return(
+        self$uploadData(ws, ds, endpoint, extension = "gpkg",
+                        configure, update, filename, charset,
+                        contentType = "")
+      )
     }
     
   )
